# Copyright (c) 2018, NVIDIA CORPORATION. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages
from setuptools.command.install import install
import subprocess
import os
import sys

def get_module_path(module_name):
    module_path = ''
    for d in sys.path:
        possible_path = os.path.join(d, module_name)
        # skip current dir as this is plugin dir
        if os.path.isdir(possible_path) and len(d) != 0:
            module_path = possible_path
            break
    return module_path

def get_tf_build_flags():
    tf_cflags = ''
    tf_lflags = ''
    try:
        import tensorflow as tensorflow
        tf_cflags=" ".join(tensorflow.sysconfig.get_compile_flags())
        tf_lflags=" ".join(tensorflow.sysconfig.get_link_flags())
    except:
        tensorflow_path = get_module_path('tensorflow')
        if tensorflow_path is not '':
            tf_cflags=" ".join(["-I" + tensorflow_path + "/include",  "-I" + tensorflow_path + "/include/external/nsync/public", "-D_GLIBCXX_USE_CXX11_ABI=0"])
            tf_lflags=" ".join(["-L" + tensorflow_path, "-ltensorflow_framework"])

    if tf_cflags is '' and tf_lflags is '':
        raise ImportError('Could not find Tensorflow. Tensorflow must be installed before installing nvidia-dali-tf-plugin')
    return (tf_cflags, tf_lflags)

def get_dali_build_flags():
    dali_cflags = ''
    dali_lflags = ''
    try:
        import nvidia.dali.sysconfig as dali_sc
        dali_lib_path = dali_sc.get_lib_dir()
        dali_cflags=" ".join(dali_sc.get_compile_flags())
        dali_lflags=" ".join(dali_sc.get_link_flags())
    except:
        dali_path = get_module_path('nvidia/dali')
        if dali_path is not '':
            dali_cflags=" ".join(["-I" + dali_path + "/include", "-D_GLIBCXX_USE_CXX11_ABI=0"])
            dali_lflags=" ".join(["-L" + dali_path, "-ldali"])
    if dali_cflags is '' and dali_lflags is '':
        raise ImportError('Could not find DALI.')
    return (dali_cflags, dali_lflags)

def get_cuda_build_flags():
    cuda_cflags = ''
    cuda_lflags = ''
    cuda_home = os.environ.get('CUDA_HOME')
    if not cuda_home:
        cuda_home = '/usr/local/cuda'
    cuda_cflags=" ".join(["-I" + cuda_home + "/include"])
    cuda_lflags=" ".join([])
    return (cuda_cflags, cuda_lflags)


class CustomInstall(install, object):
    def run(self):
        dali_cflags, dali_lflags = get_dali_build_flags()
        dali_lib_path = get_module_path('nvidia/dali')
        tf_cflags, tf_lflags = get_tf_build_flags()
        cuda_cflags, cuda_lflags = get_cuda_build_flags()

        src_path = os.path.dirname(os.path.realpath(__file__)) + '/nvidia/dali'
        plugin_src = src_path + '/plugin/daliop.cc'
        lib_path = dali_lib_path + '/plugin/libdali_tf_current.so'

        cmd = 'g++ -Wl,-R,\'$ORIGIN/..\' -std=c++11 -shared ' + plugin_src + ' -o ' + lib_path + ' -fPIC ' + dali_cflags + ' ' + tf_cflags + ' ' + cuda_cflags + ' ' + dali_lflags + ' ' + tf_lflags + ' ' + cuda_lflags + ' -O2'
        subprocess.check_call(cmd, cwd=src_path, shell=True)

        super(CustomInstall, self).run()

setup(name='nvidia-dali-tf-plugin',
      description='NVIDIA DALI Tensorflow plugin for CUDA 9.0. Git SHA: 527e2fd037e5d49af1357037daeabe68ff3f84a1',
      url='https://github.com/NVIDIA/dali',
      version='0.10.0',
      author='NVIDIA Corporation',
      license='Apache License 2.0',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires = [
          'nvidia-dali==0.10.0'
          ],
      cmdclass = {
          'install': CustomInstall
      }
     )
